#!/usr/bin/python
# vim: set et ts=4:

import sys
from charmhelpers.core.hookenv import (
    config,
    log, ERROR,
    Hooks, UnregisteredHookError,
    local_unit
)
from charmhelpers.core.host import (
    apt_update, apt_install
)
import charmhelpers.contrib.network.ovs as ovs
from utils import get_host_ip
from nvp import NVPHelper
import charmhelpers.contrib.archive as archive

hooks = Hooks()

ARCHIVE = 'nvp'


class ConfigurationError(Exception):
    pass


@hooks.hook()
def install():
    archive.install()
    archive.create_archive(ARCHIVE)
    archive.include_deb(ARCHIVE, 'payload')
    archive.configure_local_source(ARCHIVE)
    apt_update(fatal=True)


@hooks.hook('config-changed')
def config_changed():
    # Install configured packages
    apt_install(config('packages').split(), fatal=True)
    ovs.full_restart()  # Full restart to clear states

    if not config('cluster-controllers'):
        raise ConfigurationError('cluster-controllers not specified')

    controllers = config('cluster-controllers').split()
    ovs.set_manager(controllers[0])

    if not config('tz-uuid'):
        raise ConfigurationError('tz-uuid not specified')
    if not config('username') or not config('password'):
        raise ConfigurationError('username and/or password not specified')
    if not config('tz-connector-types'):
        raise ConfigurationError('tz-connector-types not specified')

    # Add the integration bridge to OVS
    ovs.add_bridge(config('integration-bridge'))

    helper = NVPHelper(host=controllers[0])
    helper.login(config('username'), config('password'))

    if not helper.transport_node_exists(local_unit()):
        helper.create_transport_node(local_unit(),
                                     get_host_ip(),
                                     ovs.get_certificate(),
                                     config('tz-connector-types').split(),
                                     config('tz-uuid'),
                                     config('integration-bridge'))
    else:
        # TODO: implement and update of connectors or tz-uuid
        pass


@hooks.hook('upgrade-charm')
def upgrade_charm():
    install()
    config_changed()


if __name__ == '__main__':
    try:
        hooks.execute(sys.argv)
    except UnregisteredHookError as e:
        log('Unknown hook {}, skipping'.format(e))
    except ConfigurationError as ce:
        log('Configuration error: {}'.format(e), level=ERROR)
        sys.exit(1)
