''' Helper for interacting with the Nicira/VMware NVP platform '''
import requests
import json


class NVPInteractionError(Exception):
    ''' Generic exception for failures in interaction with NVP '''
    pass


class NVPHelper(requests.Session):
    ''' Helper class for interacting with VMware NVP '''

    def __init__(self, host, port=443, verify=False):
        super(NVPHelper, self).__init__()
        self.nvp_url = "https://{}:{}".format(host, port)
        self.verify = verify
        self.headers.update({"content-type": "application/json"})

    def login(self, user, password):
        ''' Login to NVP controller using user and password '''
        headers = {'content-type': 'application/x-www-form-urlencoded'}
        body = {"username": user, "password": password}
        response = self.post(self._generate_url("/ws.v1/login"),
                             data=body, headers=headers)
        if response.status_code != requests.codes.ok:
            raise NVPInteractionError("login to NVP cluster failed")

    def _generate_url(self, path):
        return "{}{}".format(self.nvp_url, path)

    def get_transport_nodes(self):
        params = {"fields": "*"}
        response = self.get(self._generate_url("/ws.v1/transport-node"),
                            params=params)
        if response.status_code not in [requests.codes.ok,
                                        requests.codes.no_content]:
            raise NVPInteractionError('Unable to retrieve transport nodes: {}'
                                      .format(response.text))
        return json.loads(response.content)

    def transport_node_exists(self, tn_name):
        ''' Query NVP to see if transport node tn_name exists '''
        for tn in self.get_transport_nodes()["results"]:
            if tn["display_name"] == tn_name:
                return True
        return False

    def create_transport_node(self, tn_name, tn_ip, cert,
                              connectors, tz_uuid, int_br_id=None):
        ''' Creates a new transport node using the supplied parameters '''
        tn_obj = {
            "display_name": tn_name,
            "credential": {
                "type": "SecurityCertificateCredential",
                "client_certificate": {"pem_encoded": cert}
            },
            "transport_connectors": []
        }
        if int_br_id:
            tn_obj["integration_bridge_id"] = int_br_id
        for connector in connectors:
            tn_obj["transport_connectors"].append({
                "type": connector,
                "transport_zone_uuid": tz_uuid,
                "ip_address": tn_ip
            })
        response = self.post(self._generate_url("/ws.v1/transport-node"),
                             data=json.dumps(tn_obj))
        if response.status_code != requests.codes.created:
            raise NVPInteractionError('Unable to create transport node: {}'
                                      .format(response.text))
        return json.loads(response.content)["uuid"]

    def delete_transport_node(self, tn_name):
        ''' Delete transport node tn_name from NVP if it exists '''
        for tn in self.get_transport_nodes()["results"]:
            if tn["display_name"] == tn_name:
                response = \
                    self.delete(self._generate_url("/ws.v1/transport-node/{}"
                                                   .format(tn["uuid"])))
                if response.status_code not in [requests.codes.ok,
                                                requests.codes.no_content]:
                    raise NVPInteractionError('Unable to delete'
                                              ' transport nodes: {}'
                                              .format(response.text))
